@@ -9,7 +9,31 @@ from nautobot.utilities.tables import (
     TagColumn,
     ToggleColumn,
 )
-from .models import Circuit, CircuitType, Provider
+from .models import Circuit, CircuitType, Provider, ProviderNetwork
+
+CIRCUIT_TERMINATION_PARENT = """
+{% if value.provider_network %}
+<a href="{{ value.provider_network.get_absolute_url }}">{{ value.provider_network }}</a>
+{% elif value.site %}
+<a href="{{ value.site.get_absolute_url }}">{{ value.site }}</a>
+{% endif %}
+"""
+
+#
+# Provider Network
+#
+
+
+class ProviderNetworkTable(BaseTable):
+    pk = ToggleColumn()
+    name = tables.Column(linkify=True)
+    provider = tables.Column(linkify=True)
+    tags = TagColumn(url_name="circuits:providernetwork_list")
+
+    class Meta(BaseTable.Meta):
+        model = ProviderNetwork
+        fields = ("pk", "name", "provider", "description", "tags")
+        default_columns = ("pk", "name", "provider", "description")
 
 
 #
@@ -73,10 +97,21 @@ class CircuitTable(StatusTableMixin, BaseTable):
     cid = tables.LinkColumn(verbose_name="ID")
     provider = tables.LinkColumn(viewname="circuits:provider", args=[Accessor("provider__slug")])
     tenant = TenantColumn()
-    a_side = tables.Column(verbose_name="A Side")
-    z_side = tables.Column(verbose_name="Z Side")
     tags = TagColumn(url_name="circuits:circuit_list")
 
+    termination_a = tables.TemplateColumn(
+        template_code=CIRCUIT_TERMINATION_PARENT,
+        accessor=Accessor("termination_a"),
+        orderable=False,
+        verbose_name="Side A",
+    )
+    termination_z = tables.TemplateColumn(
+        template_code=CIRCUIT_TERMINATION_PARENT,
+        accessor=Accessor("termination_z"),
+        orderable=False,
+        verbose_name="Side Z",
+    )
+
     class Meta(BaseTable.Meta):
         model = Circuit
         fields = (
@@ -86,8 +121,8 @@ class CircuitTable(StatusTableMixin, BaseTable):
             "type",
             "status",
             "tenant",
-            "a_side",
-            "z_side",
+            "termination_a",
+            "termination_z",
             "install_date",
             "commit_rate",
             "description",
@@ -100,7 +135,7 @@ class CircuitTable(StatusTableMixin, BaseTable):
             "type",
             "status",
             "tenant",
-            "a_side",
-            "z_side",
+            "termination_a",
+            "termination_z",
             "description",
         )
