@@ -5,7 +5,7 @@ from django.db.models import Q
 from nautobot.extras.filters import (
     CustomFieldModelFilterSet,
     LocalContextFilterSet,
-    CreatedUpdatedFilterSet,
+    NautobotFilterSet,
     StatusModelFilterSetMixin,
 )
 from nautobot.extras.models import SecretsGroup
@@ -16,7 +16,10 @@ from nautobot.utilities.filters import (
     BaseFilterSet,
     MultiValueCharFilter,
     MultiValueMACAddressFilter,
+    MultiValueUUIDFilter,
     NameSlugSearchFilterSet,
+    RelatedMembershipBooleanFilter,
+    SearchFilter,
     TagFilter,
     TreeNodeMultipleChoiceFilter,
 )
@@ -108,12 +111,7 @@ __all__ = (
 )
 
 
-class RegionFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class RegionFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     parent_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Region.objects.all(),
         label="Parent region (ID)",
@@ -130,16 +128,23 @@ class RegionFilterSet(
         fields = ["id", "name", "slug", "description"]
 
 
-class SiteFilterSet(
-    BaseFilterSet,
-    TenancyFilterSet,
-    StatusModelFilterSetMixin,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class SiteFilterSet(NautobotFilterSet, TenancyFilterSet, StatusModelFilterSetMixin):
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "facility": "icontains",
+            "description": "icontains",
+            "physical_address": "icontains",
+            "shipping_address": "icontains",
+            "contact_name": "icontains",
+            "contact_phone": "icontains",
+            "contact_email": "icontains",
+            "comments": "icontains",
+            "asn": {
+                "lookup_expr": "exact",
+                "preprocessor": int,  # asn expects an int
+            },
+        },
     )
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
@@ -171,33 +176,8 @@ class SiteFilterSet(
             "contact_email",
         ]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        qs_filter = (
-            Q(name__icontains=value)
-            | Q(facility__icontains=value)
-            | Q(description__icontains=value)
-            | Q(physical_address__icontains=value)
-            | Q(shipping_address__icontains=value)
-            | Q(contact_name__icontains=value)
-            | Q(contact_phone__icontains=value)
-            | Q(contact_email__icontains=value)
-            | Q(comments__icontains=value)
-        )
-        try:
-            qs_filter |= Q(asn=int(value.strip()))
-        except ValueError:
-            pass
-        return queryset.filter(qs_filter)
 
-
-class RackGroupFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class RackGroupFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
         field_name="site__region",
@@ -237,27 +217,27 @@ class RackGroupFilterSet(
         fields = ["id", "name", "slug", "description"]
 
 
-class RackRoleFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class RackRoleFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     class Meta:
         model = RackRole
         fields = ["id", "name", "slug", "color"]
 
 
-class RackFilterSet(
-    BaseFilterSet,
-    TenancyFilterSet,
-    StatusModelFilterSetMixin,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class RackFilterSet(NautobotFilterSet, TenancyFilterSet, StatusModelFilterSetMixin):
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "facility_id": "icontains",
+            "serial": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "asset_tag": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "comments": "icontains",
+        },
     )
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
@@ -324,22 +304,15 @@ class RackFilterSet(
             "outer_unit",
         ]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(
-            Q(name__icontains=value)
-            | Q(facility_id__icontains=value)
-            | Q(serial__icontains=value.strip())
-            | Q(asset_tag__icontains=value.strip())
-            | Q(comments__icontains=value)
-        )
 
-
-class RackReservationFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet, TenancyFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class RackReservationFilterSet(NautobotFilterSet, TenancyFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "rack__name": "icontains",
+            "rack__facility_id": "icontains",
+            "user__username": "icontains",
+            "description": "icontains",
+        },
     )
     rack_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Rack.objects.all(),
@@ -385,32 +358,21 @@ class RackReservationFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFie
         model = RackReservation
         fields = ["id", "created"]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(
-            Q(rack__name__icontains=value)
-            | Q(rack__facility_id__icontains=value)
-            | Q(user__username__icontains=value)
-            | Q(description__icontains=value)
-        )
 
-
-class ManufacturerFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class ManufacturerFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     class Meta:
         model = Manufacturer
         fields = ["id", "name", "slug", "description"]
 
 
-class DeviceTypeFilterSet(BaseFilterSet, CustomFieldModelFilterSet, CreatedUpdatedFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class DeviceTypeFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "manufacturer__name": "icontains",
+            "model": "icontains",
+            "part_number": "icontains",
+            "comments": "icontains",
+        },
     )
     manufacturer_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Manufacturer.objects.all(),
@@ -464,16 +426,6 @@ class DeviceTypeFilterSet(BaseFilterSet, CustomFieldModelFilterSet, CreatedUpdat
             "subdevice_role",
         ]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(
-            Q(manufacturer__name__icontains=value)
-            | Q(model__icontains=value)
-            | Q(part_number__icontains=value)
-            | Q(comments__icontains=value)
-        )
-
     def _console_ports(self, queryset, name, value):
         return queryset.exclude(consoleporttemplates__isnull=value)
 
@@ -552,23 +504,13 @@ class DeviceBayTemplateFilterSet(BaseFilterSet, DeviceTypeComponentFilterSet):
         fields = ["id", "name"]
 
 
-class DeviceRoleFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class DeviceRoleFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     class Meta:
         model = DeviceRole
         fields = ["id", "name", "slug", "color", "vm_role"]
 
 
-class PlatformFilterSet(
-    BaseFilterSet,
-    NameSlugSearchFilterSet,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
+class PlatformFilterSet(NautobotFilterSet, NameSlugSearchFilterSet):
     manufacturer_id = django_filters.ModelMultipleChoiceFilter(
         field_name="manufacturer",
         queryset=Manufacturer.objects.all(),
@@ -586,17 +528,24 @@ class PlatformFilterSet(
         fields = ["id", "name", "slug", "napalm_driver", "description"]
 
 
-class DeviceFilterSet(
-    BaseFilterSet,
-    TenancyFilterSet,
-    LocalContextFilterSet,
-    StatusModelFilterSetMixin,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
-):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class DeviceFilterSet(NautobotFilterSet, TenancyFilterSet, LocalContextFilterSet, StatusModelFilterSetMixin):
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "serial": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "inventoryitems__serial": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "asset_tag": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "comments": "icontains",
+        },
     )
     manufacturer_id = django_filters.ModelMultipleChoiceFilter(
         field_name="device_type__manufacturer",
@@ -707,37 +656,53 @@ class DeviceFilterSet(
         queryset=VirtualChassis.objects.all(),
         label="Virtual chassis (ID)",
     )
-    virtual_chassis_member = django_filters.BooleanFilter(
-        method="_virtual_chassis_member", label="Is a virtual chassis member"
+    is_virtual_chassis_member = RelatedMembershipBooleanFilter(
+        field_name="virtual_chassis",
+        label="Is a virtual chassis member",
     )
-    console_ports = django_filters.BooleanFilter(
-        method="_console_ports",
+    virtual_chassis_member = is_virtual_chassis_member
+    has_console_ports = RelatedMembershipBooleanFilter(
+        field_name="consoleports",
         label="Has console ports",
     )
-    console_server_ports = django_filters.BooleanFilter(
-        method="_console_server_ports",
+    console_ports = has_console_ports
+    has_console_server_ports = RelatedMembershipBooleanFilter(
+        field_name="consoleserverports",
         label="Has console server ports",
     )
-    power_ports = django_filters.BooleanFilter(
-        method="_power_ports",
+    console_server_ports = has_console_server_ports
+    has_power_ports = RelatedMembershipBooleanFilter(
+        field_name="powerports",
         label="Has power ports",
     )
-    power_outlets = django_filters.BooleanFilter(
-        method="_power_outlets",
+    power_ports = has_power_ports
+    has_power_outlets = RelatedMembershipBooleanFilter(
+        field_name="poweroutlets",
         label="Has power outlets",
     )
-    interfaces = django_filters.BooleanFilter(
-        method="_interfaces",
+    power_outlets = has_power_outlets
+    has_interfaces = RelatedMembershipBooleanFilter(
+        field_name="interfaces",
         label="Has interfaces",
     )
+    interfaces = has_interfaces
     pass_through_ports = django_filters.BooleanFilter(
         method="_pass_through_ports",
         label="Has pass-through ports",
     )
-    device_bays = django_filters.BooleanFilter(
-        method="_device_bays",
+    has_front_ports = RelatedMembershipBooleanFilter(
+        field_name="frontports",
+        label="Has front ports",
+    )
+    has_rear_ports = RelatedMembershipBooleanFilter(
+        field_name="rearports",
+        label="Has rear ports",
+    )
+    has_device_bays = RelatedMembershipBooleanFilter(
+        field_name="devicebays",
         label="Has device bays",
     )
+    device_bays = has_device_bays
     tag = TagFilter()
 
     class Meta:
@@ -752,52 +717,24 @@ class DeviceFilterSet(
             "vc_priority",
         ]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(
-            Q(name__icontains=value)
-            | Q(serial__icontains=value.strip())
-            | Q(inventoryitems__serial__icontains=value.strip())
-            | Q(asset_tag__icontains=value.strip())
-            | Q(comments__icontains=value)
-        ).distinct()
-
     def _has_primary_ip(self, queryset, name, value):
         params = Q(primary_ip4__isnull=False) | Q(primary_ip6__isnull=False)
         if value:
             return queryset.filter(params)
         return queryset.exclude(params)
 
-    def _virtual_chassis_member(self, queryset, name, value):
-        return queryset.exclude(virtual_chassis__isnull=value)
-
-    def _console_ports(self, queryset, name, value):
-        return queryset.exclude(consoleports__isnull=value)
-
-    def _console_server_ports(self, queryset, name, value):
-        return queryset.exclude(consoleserverports__isnull=value)
-
-    def _power_ports(self, queryset, name, value):
-        return queryset.exclude(powerports__isnull=value)
-
-    def _power_outlets(self, queryset, name, value):
-        return queryset.exclude(poweroutlets__isnull=value)
-
-    def _interfaces(self, queryset, name, value):
-        return queryset.exclude(interfaces__isnull=value)
-
+    # 2.0 TODO: Remove me and `pass_through_ports` in exchange for `has_(front|rear)_ports`.
     def _pass_through_ports(self, queryset, name, value):
         return queryset.exclude(frontports__isnull=value, rearports__isnull=value)
 
-    def _device_bays(self, queryset, name, value):
-        return queryset.exclude(devicebays__isnull=value)
-
 
 class DeviceComponentFilterSet(CustomFieldModelFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "label": "icontains",
+            "description": "icontains",
+        },
     )
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
@@ -835,11 +772,6 @@ class DeviceComponentFilterSet(CustomFieldModelFilterSet):
     )
     tag = TagFilter()
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(Q(name__icontains=value) | Q(label__icontains=value) | Q(description__icontains=value))
-
 
 class CableTerminationFilterSet(django_filters.FilterSet):
     cabled = django_filters.BooleanFilter(field_name="cable", lookup_expr="isnull", exclude=True)
@@ -913,10 +845,6 @@ class InterfaceFilterSet(
     CableTerminationFilterSet,
     PathEndpointFilterSet,
 ):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
-    )
     # Override device and device_id filters from DeviceComponentFilterSet to match against any peer virtual chassis
     # members
     device = MultiValueCharFilter(
@@ -924,7 +852,7 @@ class InterfaceFilterSet(
         field_name="name",
         label="Device (name)",
     )
-    device_id = MultiValueCharFilter(
+    device_id = MultiValueUUIDFilter(
         method="filter_device_id",
         field_name="pk",
         label="Device (ID)",
@@ -1018,9 +946,20 @@ class DeviceBayFilterSet(BaseFilterSet, DeviceComponentFilterSet):
 
 
 class InventoryItemFilterSet(BaseFilterSet, DeviceComponentFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "part_id": "icontains",
+            "serial": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "asset_tag": {
+                "lookup_expr": "icontains",
+                "preprocessor": str.strip,
+            },
+            "description": "icontains",
+        },
     )
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
@@ -1075,23 +1014,14 @@ class InventoryItemFilterSet(BaseFilterSet, DeviceComponentFilterSet):
         model = InventoryItem
         fields = ["id", "name", "part_id", "asset_tag", "discovered"]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        qs_filter = (
-            Q(name__icontains=value)
-            | Q(part_id__icontains=value)
-            | Q(serial__icontains=value)
-            | Q(asset_tag__icontains=value)
-            | Q(description__icontains=value)
-        )
-        return queryset.filter(qs_filter)
 
-
-class VirtualChassisFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
+class VirtualChassisFilterSet(NautobotFilterSet):
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "members__name": "icontains",
+            "domain": "icontains",
+        },
     )
     master_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Device.objects.all(),
@@ -1144,27 +1074,18 @@ class VirtualChassisFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFiel
         model = VirtualChassis
         fields = ["id", "domain", "name"]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        qs_filter = Q(name__icontains=value) | Q(members__name__icontains=value) | Q(domain__icontains=value)
-        return queryset.filter(qs_filter)
 
-
-class CableFilterSet(StatusModelFilterSetMixin, BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
-    )
+class CableFilterSet(NautobotFilterSet, StatusModelFilterSetMixin):
+    q = SearchFilter(filter_predicates={"label": "icontains"})
     type = django_filters.MultipleChoiceFilter(choices=CableTypeChoices)
     color = django_filters.MultipleChoiceFilter(choices=ColorChoices)
-    device_id = MultiValueCharFilter(method="filter_device", label="Device (ID)")
+    device_id = MultiValueUUIDFilter(method="filter_device", label="Device (ID)")
     device = MultiValueCharFilter(method="filter_device", field_name="device__name", label="Device (name)")
-    rack_id = MultiValueCharFilter(method="filter_device", field_name="device__rack_id", label="Rack (ID)")
+    rack_id = MultiValueUUIDFilter(method="filter_device", field_name="device__rack_id", label="Rack (ID)")
     rack = MultiValueCharFilter(method="filter_device", field_name="device__rack__name", label="Rack (name)")
-    site_id = MultiValueCharFilter(method="filter_device", field_name="device__site_id", label="Site (ID)")
+    site_id = MultiValueUUIDFilter(method="filter_device", field_name="device__site_id", label="Site (ID)")
     site = MultiValueCharFilter(method="filter_device", field_name="device__site__slug", label="Site (name)")
-    tenant_id = MultiValueCharFilter(method="filter_device", field_name="device__tenant_id", label="Tenant (ID)")
+    tenant_id = MultiValueUUIDFilter(method="filter_device", field_name="device__tenant_id", label="Tenant (ID)")
     tenant = MultiValueCharFilter(method="filter_device", field_name="device__tenant__slug", label="Tenant (name)")
     tag = TagFilter()
 
@@ -1172,11 +1093,6 @@ class CableFilterSet(StatusModelFilterSetMixin, BaseFilterSet, CreatedUpdatedFil
         model = Cable
         fields = ["id", "label", "length", "length_unit"]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        return queryset.filter(label__icontains=value)
-
     def filter_device(self, queryset, name, value):
         queryset = queryset.filter(
             Q(**{"_termination_a_{}__in".format(name): value}) | Q(**{"_termination_b_{}__in".format(name): value})
@@ -1201,7 +1117,7 @@ class ConsoleConnectionFilterSet(ConnectionFilterSet, BaseFilterSet):
         method="filter_site",
         label="Site (slug)",
     )
-    device_id = MultiValueCharFilter(method="filter_device", label="Device (ID)")
+    device_id = MultiValueUUIDFilter(method="filter_device", label="Device (ID)")
     device = MultiValueCharFilter(method="filter_device", field_name="device__name", label="Device (name)")
 
     class Meta:
@@ -1214,7 +1130,7 @@ class PowerConnectionFilterSet(ConnectionFilterSet, BaseFilterSet):
         method="filter_site",
         label="Site (slug)",
     )
-    device_id = MultiValueCharFilter(method="filter_device", label="Device (ID)")
+    device_id = MultiValueUUIDFilter(method="filter_device", label="Device (ID)")
     device = MultiValueCharFilter(method="filter_device", field_name="device__name", label="Device (name)")
 
     class Meta:
@@ -1227,7 +1143,7 @@ class InterfaceConnectionFilterSet(ConnectionFilterSet, BaseFilterSet):
         method="filter_site",
         label="Site (slug)",
     )
-    device_id = MultiValueCharFilter(method="filter_device", label="Device (ID)")
+    device_id = MultiValueUUIDFilter(method="filter_device", label="Device (ID)")
     device = MultiValueCharFilter(method="filter_device", field_name="device__name", label="Device (name)")
 
     class Meta:
@@ -1235,11 +1151,8 @@ class InterfaceConnectionFilterSet(ConnectionFilterSet, BaseFilterSet):
         fields = []
 
 
-class PowerPanelFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldModelFilterSet):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
-    )
+class PowerPanelFilterSet(NautobotFilterSet):
+    q = SearchFilter(filter_predicates={"name": "icontains"})
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
         field_name="site__region",
@@ -1275,25 +1188,11 @@ class PowerPanelFilterSet(BaseFilterSet, CreatedUpdatedFilterSet, CustomFieldMod
         model = PowerPanel
         fields = ["id", "name"]
 
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        qs_filter = Q(name__icontains=value)
-        return queryset.filter(qs_filter)
-
 
 class PowerFeedFilterSet(
-    BaseFilterSet,
-    CableTerminationFilterSet,
-    PathEndpointFilterSet,
-    StatusModelFilterSetMixin,
-    CustomFieldModelFilterSet,
-    CreatedUpdatedFilterSet,
+    NautobotFilterSet, CableTerminationFilterSet, PathEndpointFilterSet, StatusModelFilterSetMixin
 ):
-    q = django_filters.CharFilter(
-        method="search",
-        label="Search",
-    )
+    q = SearchFilter(filter_predicates={"name": "icontains", "comments": "icontains"})
     region_id = TreeNodeMultipleChoiceFilter(
         queryset=Region.objects.all(),
         field_name="power_panel__site__region",
@@ -1342,9 +1241,3 @@ class PowerFeedFilterSet(
             "amperage",
             "max_utilization",
         ]
-
-    def search(self, queryset, name, value):
-        if not value.strip():
-            return queryset
-        qs_filter = Q(name__icontains=value) | Q(comments__icontains=value)
-        return queryset.filter(qs_filter)
