@@ -1,8 +1,14 @@
 from django.core.exceptions import ValidationError
 from django.test import TestCase
+from django.contrib.contenttypes.models import ContentType
 
-from nautobot.circuits.models import Circuit, CircuitTermination, CircuitType, Provider
-from nautobot.dcim.choices import DeviceFaceChoices, PowerOutletFeedLegChoices, InterfaceTypeChoices, PortTypeChoices
+from nautobot.circuits.models import Circuit, CircuitTermination, CircuitType, Provider, ProviderNetwork
+from nautobot.dcim.choices import (
+    DeviceFaceChoices,
+    InterfaceTypeChoices,
+    PortTypeChoices,
+    PowerOutletFeedLegChoices,
+)
 from nautobot.dcim.models import (
     Cable,
     ConsolePort,
@@ -30,10 +36,113 @@ from nautobot.dcim.models import (
     RearPortTemplate,
     Site,
 )
-from nautobot.extras.models import Status
+from nautobot.extras.choices import CustomFieldTypeChoices
+from nautobot.extras.models import CustomField, Status
 from nautobot.tenancy.models import Tenant
 
 
+class CableLengthTestCase(TestCase):
+    def setUp(self):
+        self.site = Site.objects.create(name="Test Site 1", slug="test-site-1")
+        self.manufacturer = Manufacturer.objects.create(name="Test Manufacturer 1", slug="test-manufacturer-1")
+        self.devicetype = DeviceType.objects.create(
+            manufacturer=self.manufacturer,
+            model="Test Device Type 1",
+            slug="test-device-type-1",
+        )
+        self.devicerole = DeviceRole.objects.create(
+            name="Test Device Role 1", slug="test-device-role-1", color="ff0000"
+        )
+        self.device1 = Device.objects.create(
+            device_type=self.devicetype,
+            device_role=self.devicerole,
+            name="TestDevice1",
+            site=self.site,
+        )
+        self.device2 = Device.objects.create(
+            device_type=self.devicetype,
+            device_role=self.devicerole,
+            name="TestDevice2",
+            site=self.site,
+        )
+        self.status = Status.objects.get_for_model(Cable).get(slug="connected")
+
+    def test_cable_validated_save(self):
+        interface1 = Interface.objects.create(device=self.device1, name="eth0")
+        interface2 = Interface.objects.create(device=self.device2, name="eth0")
+        cable = Cable(
+            termination_a=interface1,
+            termination_b=interface2,
+            length_unit="ft",
+            length=1,
+            status=self.status,
+        )
+        cable.validated_save()
+        cable.validated_save()
+
+    def test_cable_full_clean(self):
+        interface3 = Interface.objects.create(device=self.device1, name="eth1")
+        interface4 = Interface.objects.create(device=self.device2, name="eth1")
+        cable = Cable(
+            termination_a=interface3,
+            termination_b=interface4,
+            length_unit="in",
+            length=1,
+            status=self.status,
+        )
+        cable.length = 2
+        cable.save()
+        cable.full_clean()
+
+
+class InterfaceTemplateCustomFieldTestCase(TestCase):
+    def test_instantiate_model(self):
+        """
+        Check that all _custom_field_data is present and all customfields are filled with the correct default values.
+        """
+        statuses = Status.objects.get_for_model(Device)
+        site = Site.objects.create(name="Site 1", slug="site-1")
+        manufacturer = Manufacturer.objects.create(name="Acme", slug="acme")
+        device_role = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1", color="ff0000")
+        custom_fields = [
+            CustomField.objects.create(type=CustomFieldTypeChoices.TYPE_TEXT, name="field_1", default="value_1"),
+            CustomField.objects.create(type=CustomFieldTypeChoices.TYPE_TEXT, name="field_2", default="value_2"),
+            CustomField.objects.create(type=CustomFieldTypeChoices.TYPE_TEXT, name="field_3", default="value_3"),
+        ]
+        for custom_field in custom_fields:
+            custom_field.content_types.set([ContentType.objects.get_for_model(Interface)])
+        device_type = DeviceType.objects.create(manufacturer=manufacturer, model="FrameForwarder 2048", slug="ff2048")
+        interface_template_1 = InterfaceTemplate.objects.create(
+            device_type=device_type,
+            name="Test_Template_1",
+            type=InterfaceTypeChoices.TYPE_1GE_FIXED,
+            mgmt_only=True,
+        )
+        interface_template_2 = InterfaceTemplate.objects.create(
+            device_type=device_type,
+            name="Test_Template_2",
+            type=InterfaceTypeChoices.TYPE_1GE_FIXED,
+            mgmt_only=True,
+        )
+        interface_templates = [interface_template_1, interface_template_2]
+        device_type.interfacetemplates.set(interface_templates)
+        # instantiate_model() is run when device is created
+        device = Device.objects.create(
+            device_type=device_type,
+            device_role=device_role,
+            status=statuses[0],
+            name="Test Device",
+            site=site,
+        )
+        interfaces = device.interfaces.all()
+        self.assertEqual(Interface.objects.get(pk=interfaces[0].pk).cf["field_1"], "value_1")
+        self.assertEqual(Interface.objects.get(pk=interfaces[0].pk).cf["field_2"], "value_2")
+        self.assertEqual(Interface.objects.get(pk=interfaces[0].pk).cf["field_3"], "value_3")
+        self.assertEqual(Interface.objects.get(pk=interfaces[1].pk).cf["field_1"], "value_1")
+        self.assertEqual(Interface.objects.get(pk=interfaces[1].pk).cf["field_2"], "value_2")
+        self.assertEqual(Interface.objects.get(pk=interfaces[1].pk).cf["field_3"], "value_3")
+
+
 class RackGroupTestCase(TestCase):
     def test_change_rackgroup_site(self):
         """
@@ -427,49 +536,54 @@ class CableTestCase(TestCase):
         self.cable.save()
 
         self.power_port1 = PowerPort.objects.create(device=self.device2, name="psu1")
-        self.patch_pannel = Device.objects.create(
+        self.patch_panel = Device.objects.create(
             device_type=devicetype,
             device_role=devicerole,
-            name="TestPatchPannel",
+            name="TestPatchPanel",
             site=site,
         )
-        self.rear_port1 = RearPort.objects.create(device=self.patch_pannel, name="RP1", type="8p8c")
+        self.rear_port1 = RearPort.objects.create(device=self.patch_panel, name="RP1", type="8p8c")
         self.front_port1 = FrontPort.objects.create(
-            device=self.patch_pannel,
+            device=self.patch_panel,
             name="FP1",
             type="8p8c",
             rear_port=self.rear_port1,
             rear_port_position=1,
         )
-        self.rear_port2 = RearPort.objects.create(device=self.patch_pannel, name="RP2", type="8p8c", positions=2)
+        self.rear_port2 = RearPort.objects.create(device=self.patch_panel, name="RP2", type="8p8c", positions=2)
         self.front_port2 = FrontPort.objects.create(
-            device=self.patch_pannel,
+            device=self.patch_panel,
             name="FP2",
             type="8p8c",
             rear_port=self.rear_port2,
             rear_port_position=1,
         )
-        self.rear_port3 = RearPort.objects.create(device=self.patch_pannel, name="RP3", type="8p8c", positions=3)
+        self.rear_port3 = RearPort.objects.create(device=self.patch_panel, name="RP3", type="8p8c", positions=3)
         self.front_port3 = FrontPort.objects.create(
-            device=self.patch_pannel,
+            device=self.patch_panel,
             name="FP3",
             type="8p8c",
             rear_port=self.rear_port3,
             rear_port_position=1,
         )
-        self.rear_port4 = RearPort.objects.create(device=self.patch_pannel, name="RP4", type="8p8c", positions=3)
+        self.rear_port4 = RearPort.objects.create(device=self.patch_panel, name="RP4", type="8p8c", positions=3)
         self.front_port4 = FrontPort.objects.create(
-            device=self.patch_pannel,
+            device=self.patch_panel,
             name="FP4",
             type="8p8c",
             rear_port=self.rear_port4,
             rear_port_position=1,
         )
         self.provider = Provider.objects.create(name="Provider 1", slug="provider-1")
+        provider_network = ProviderNetwork.objects.create(name="Provider Network 1", provider=self.provider)
         self.circuittype = CircuitType.objects.create(name="Circuit Type 1", slug="circuit-type-1")
-        self.circuit = Circuit.objects.create(provider=self.provider, type=self.circuittype, cid="1")
-        self.circuittermination1 = CircuitTermination.objects.create(circuit=self.circuit, site=site, term_side="A")
-        self.circuittermination2 = CircuitTermination.objects.create(circuit=self.circuit, site=site, term_side="Z")
+        self.circuit1 = Circuit.objects.create(provider=self.provider, type=self.circuittype, cid="1")
+        self.circuit2 = Circuit.objects.create(provider=self.provider, type=self.circuittype, cid="2")
+        self.circuittermination1 = CircuitTermination.objects.create(circuit=self.circuit1, site=site, term_side="A")
+        self.circuittermination2 = CircuitTermination.objects.create(circuit=self.circuit1, site=site, term_side="Z")
+        self.circuittermination3 = CircuitTermination.objects.create(
+            circuit=self.circuit2, provider_network=provider_network, term_side="Z"
+        )
 
     def test_cable_creation(self):
         """
@@ -539,6 +653,14 @@ class CableTestCase(TestCase):
         with self.assertRaises(ValidationError):
             cable.clean()
 
+    def test_cable_cannot_terminate_to_a_provider_network_circuittermination(self):
+        """
+        Neither side of a cable can be terminated to a CircuitTermination which is attached to a Provider Network
+        """
+        cable = Cable(termination_a=self.interface3, termination_b=self.circuittermination3)
+        with self.assertRaises(ValidationError):
+            cable.clean()
+
     def test_rearport_connections(self):
         """
         Test various combinations of RearPort connections.
