@@ -2,6 +2,14 @@ from .change_logging import ChangeLoggedModel, ObjectChange
 from .statuses import Status, StatusField, StatusModel
 from .customfields import ComputedField, CustomField, CustomFieldChoice, CustomFieldModel
 from .datasources import GitRepository
+from .groups import DynamicGroup
+from .jobs import (
+    Job,
+    JobLogEntry,
+    JobResult,
+    ScheduledJob,
+    ScheduledJobs,
+)
 from .models import (
     ConfigContext,
     ConfigContextModel,
@@ -13,17 +21,13 @@ from .models import (
     GraphQLQuery,
     HealthCheckTestModel,
     ImageAttachment,
-    Job,
-    JobLogEntry,
-    JobResult,
-    ScheduledJob,
-    ScheduledJobs,
     Webhook,
 )
-from .relationships import Relationship, RelationshipModel, RelationshipAssociation
+from .relationships import Relationship, RelationshipAssociation, RelationshipModel
 from .secrets import Secret, SecretsGroup, SecretsGroupAssociation
 from .tags import Tag, TaggedItem
 
+
 __all__ = (
     "ChangeLoggedModel",
     "ComputedField",
@@ -34,6 +38,7 @@ __all__ = (
     "CustomFieldChoice",
     "CustomFieldModel",
     "CustomLink",
+    "DynamicGroup",
     "ExportTemplate",
     "FileAttachment",
     "FileProxy",
