@@ -3,11 +3,11 @@ from django.contrib.contenttypes.models import ContentType
 from nautobot.dcim.models import Device, DeviceRole, DeviceType, Manufacturer, Site
 from nautobot.extras.models import ConfigContext, ConfigContextSchema, Status
 from nautobot.utilities.choices import ColorChoices
-from nautobot.utilities.testing.integration import SplinterTestCase
+from nautobot.utilities.testing.integration import SeleniumTestCase
 from nautobot.virtualization.models import Cluster, ClusterType, VirtualMachine
 
 
-class ConfigContextSchemaTestCase(SplinterTestCase):
+class ConfigContextSchemaTestCase(SeleniumTestCase):
     """
     Integration tests for the ConfigContextSchema model
     """
@@ -143,7 +143,8 @@ class ConfigContextSchemaTestCase(SplinterTestCase):
         )  # 3 rows (config context, device, virtual machine)
         for row in self.browser.find_by_xpath("//tbody/tr"):
             self.assertEqual(
-                row.find_by_tag("td")[-2].html, '<span class="text-success"><i class="mdi mdi-check-bold"></i></span>'
+                row.find_by_tag("td")[-2].html,
+                '<span class="text-success"><i class="mdi mdi-check-bold" title="Yes"></i></span>',
             )
 
         # Edit the schema
@@ -165,7 +166,7 @@ class ConfigContextSchemaTestCase(SplinterTestCase):
         for row in self.browser.find_by_xpath("//tbody/tr"):
             self.assertEqual(
                 row.find_by_tag("td")[-2].html,
-                '<span class="text-danger"><i class="mdi mdi-close-thick"></i>123 is not of type \'string\'</span>',
+                '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
             )
 
         # Edit the device local context data and redirect back to the validation tab
@@ -181,15 +182,15 @@ class ConfigContextSchemaTestCase(SplinterTestCase):
         # Config context still fails
         self.assertEqual(
             self.browser.find_by_xpath("//tbody/tr")[0].find_by_tag("td")[-2].html,
-            '<span class="text-danger"><i class="mdi mdi-close-thick"></i>123 is not of type \'string\'</span>',
+            '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
         )
         # Device now passes
         self.assertEqual(
             self.browser.find_by_xpath("//tbody/tr")[1].find_by_tag("td")[-2].html,
-            '<span class="text-success"><i class="mdi mdi-check-bold"></i></span>',
+            '<span class="text-success"><i class="mdi mdi-check-bold" title="Yes"></i></span>',
         )
         # Virtual machine still fails
         self.assertEqual(
             self.browser.find_by_xpath("//tbody/tr")[2].find_by_tag("td")[-2].html,
-            '<span class="text-danger"><i class="mdi mdi-close-thick"></i>123 is not of type \'string\'</span>',
+            '<span class="text-danger"><i class="mdi mdi-close-thick" title="No"></i></span><span class="text-danger">123 is not of type \'string\'</span>',
         )
