@@ -3,10 +3,13 @@ from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
 from django.urls import reverse
 from django.test import override_settings, tag
+from django.utils.text import slugify
 from rest_framework import status
 from rest_framework.test import APIClient, APITransactionTestCase as _APITransactionTestCase
 
 from nautobot.users.models import ObjectPermission, Token
+from nautobot.extras.choices import ObjectChangeActionChoices
+from nautobot.extras.models import ObjectChange
 from .utils import disable_warnings
 from .views import ModelTestCase
 
@@ -32,10 +35,12 @@ class APITestCase(ModelTestCase):
 
     client_class: Test client class
     view_namespace: Namespace for API views. If None, the model's app_label will be used.
+    api_version: Specific API version to test. Leave unset to test the default behavior. Override with set_api_version()
     """
 
     client_class = APIClient
     view_namespace = None
+    api_version = None
 
     def setUp(self):
         """
@@ -46,6 +51,15 @@ class APITestCase(ModelTestCase):
         self.add_permissions(*self.user_permissions)
         self.token = Token.objects.create(user=self.user)
         self.header = {"HTTP_AUTHORIZATION": "Token {}".format(self.token.key)}
+        if self.api_version:
+            self.set_api_version(self.api_version)
+
+    def set_api_version(self, api_version):
+        """Set or unset a specific API version for requests in this test case."""
+        if api_version is None:
+            self.header["HTTP_ACCEPT"] = "application/json"
+        else:
+            self.header["HTTP_ACCEPT"] = f"application/json; version={api_version}"
 
     def _get_view_namespace(self):
         if self.view_namespace:
@@ -153,6 +167,8 @@ class APIViewTestCases:
             else:
                 response = self.client.get(url, **self.header)
                 self.assertHttpStatus(response, status.HTTP_200_OK)
+                self.assertIsInstance(response.data, dict)
+                self.assertIn("results", response.data)
                 self.assertEqual(len(response.data["results"]), self._get_queryset().count())
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
@@ -164,6 +180,9 @@ class APIViewTestCases:
             url = f"{self._get_list_url()}?brief=1"
             response = self.client.get(url, **self.header)
 
+            self.assertHttpStatus(response, status.HTTP_200_OK)
+            self.assertIsInstance(response.data, dict)
+            self.assertIn("results", response.data)
             self.assertEqual(len(response.data["results"]), self._get_queryset().count())
             self.assertEqual(sorted(response.data["results"][0]), self.brief_fields)
 
@@ -203,6 +222,8 @@ class APIViewTestCases:
             # Try GET to permitted objects
             response = self.client.get(self._get_list_url(), **self.header)
             self.assertHttpStatus(response, status.HTTP_200_OK)
+            self.assertIsInstance(response.data, dict)
+            self.assertIn("results", response.data)
             self.assertEqual(len(response.data["results"]), 2)
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
@@ -224,9 +245,17 @@ class APIViewTestCases:
 
             response = self.client.options(self._get_list_url(), **self.header)
             self.assertHttpStatus(response, status.HTTP_200_OK)
+            data = response.json()
+
+            self.assertIn("actions", data)
 
             # Grab any field that has choices defined (fields with enums)
-            field_choices = {k: v["choices"] for k, v in response.json()["actions"]["POST"].items() if "choices" in v}
+            if "POST" in data["actions"]:
+                field_choices = {k: v["choices"] for k, v in data["actions"]["POST"].items() if "choices" in v}
+            elif "PUT" in data["actions"]:  # JobModelViewSet supports editing but not creation
+                field_choices = {k: v["choices"] for k, v in data["actions"]["PUT"].items() if "choices" in v}
+            else:
+                self.fail(f"Neither PUT nor POST are available actions in: {data['actions']}")
 
             # Will successfully assert if field_choices has entries and will not fail if model as no enum choices
             # Broken down to provide better failure messages
@@ -250,9 +279,17 @@ class APIViewTestCases:
 
             response = self.client.options(self._get_list_url(), **self.header)
             self.assertHttpStatus(response, status.HTTP_200_OK)
+            data = response.json()
+
+            self.assertIn("actions", data)
 
             # Grab any field name that has choices defined (fields with enums)
-            field_choices = {k for k, v in response.json()["actions"]["POST"].items() if "choices" in v}
+            if "POST" in data["actions"]:
+                field_choices = {k for k, v in data["actions"]["POST"].items() if "choices" in v}
+            elif "PUT" in data["actions"]:  # JobModelViewSet supports editing but not creation
+                field_choices = {k for k, v in data["actions"]["PUT"].items() if "choices" in v}
+            else:
+                self.fail(f"Neither PUT nor POST are available actions in: {data['actions']}")
 
             self.assertEqual(set(self.choices_fields), field_choices)
 
@@ -271,7 +308,12 @@ class APIViewTestCases:
             self.user.save()
 
             response = self.client.options(self._get_list_url(), **self.header)
-            actions = response.json()["actions"]["POST"]
+            data = response.json()
+
+            self.assertIn("actions", data)
+            self.assertIn("POST", data["actions"])
+
+            actions = data["actions"]["POST"]
             choices = actions["status"]["choices"]
 
             # Import Status here to avoid circular import issues w/ test utilities.
@@ -285,6 +327,7 @@ class APIViewTestCases:
     class CreateObjectViewTestCase(APITestCase):
         create_data = []
         validation_excluded_fields = []
+        slug_source = None
 
         def test_create_object_without_permission(self):
             """
@@ -312,12 +355,26 @@ class APIViewTestCases:
                 response = self.client.post(self._get_list_url(), create_data, format="json", **self.header)
                 self.assertHttpStatus(response, status.HTTP_201_CREATED)
                 self.assertEqual(self._get_queryset().count(), initial_count + i + 1)
+                instance = self._get_queryset().get(pk=response.data["id"])
                 self.assertInstanceEqual(
-                    self._get_queryset().get(pk=response.data["id"]),
+                    instance,
                     create_data,
                     exclude=self.validation_excluded_fields,
                     api=True,
                 )
+                # Check if Slug field is automatically created
+                if self.slug_source is not None and "slug" not in create_data:
+                    object = self._get_queryset().get(pk=response.data["id"])
+                    expected_slug = slugify(getattr(object, self.slug_source))
+                    self.assertEqual(object.slug, expected_slug)
+
+                # Verify ObjectChange creation
+                if hasattr(self.model, "to_objectchange"):
+                    objectchanges = ObjectChange.objects.filter(
+                        changed_object_type=ContentType.objects.get_for_model(instance), changed_object_id=instance.pk
+                    )
+                    self.assertEqual(len(objectchanges), 1)
+                    self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_CREATE)
 
         def test_bulk_create_objects(self):
             """
@@ -349,6 +406,10 @@ class APIViewTestCases:
                     exclude=self.validation_excluded_fields,
                     api=True,
                 )
+                if self.slug_source is not None and "slug" not in self.create_data[i]:
+                    object = self._get_queryset().get(pk=obj["id"])
+                    expected_slug = slugify(getattr(object, self.slug_source))
+                    self.assertEqual(object.slug, expected_slug)
 
     class UpdateObjectViewTestCase(APITestCase):
         update_data = {}
@@ -386,6 +447,14 @@ class APIViewTestCases:
             instance.refresh_from_db()
             self.assertInstanceEqual(instance, update_data, exclude=self.validation_excluded_fields, api=True)
 
+            # Verify ObjectChange creation
+            if hasattr(self.model, "to_objectchange"):
+                objectchanges = ObjectChange.objects.filter(
+                    changed_object_type=ContentType.objects.get_for_model(instance), changed_object_id=instance.pk
+                )
+                self.assertEqual(len(objectchanges), 1)
+                self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_UPDATE)
+
         def test_bulk_update_objects(self):
             """
             PATCH a set of objects in a single request.
@@ -449,6 +518,14 @@ class APIViewTestCases:
             self.assertHttpStatus(response, status.HTTP_204_NO_CONTENT)
             self.assertFalse(self._get_queryset().filter(pk=instance.pk).exists())
 
+            # Verify ObjectChange creation
+            if hasattr(self.model, "to_objectchange"):
+                objectchanges = ObjectChange.objects.filter(
+                    changed_object_type=ContentType.objects.get_for_model(instance), changed_object_id=instance.pk
+                )
+                self.assertEqual(len(objectchanges), 1)
+                self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_DELETE)
+
         def test_bulk_delete_objects(self):
             """
             DELETE a set of objects in a single request.
